/*
 * readlineSync
 * https://github.com/anseki/readline-sync
 *
 * Copyright (c) 2014 anseki
 * Licensed under the MIT license.
 */

'use strict';

var promptText = '> ',
  encoding = 'utf8',
  BUF_SIZE = 256,
  fs = require('fs'),
  stdin = process.stdin,
  stdout = process.stdout,
  buffer = new Buffer(BUF_SIZE),
  useShell = true, tempdir;

function _readlineSync(display) {
  var input = '', rsize, err;

  if (display) { stdout.write(display, encoding); }

  stdin.resume();
  while (true) {
    rsize = 0;

    try {
      rsize = fs.readSync(stdin.fd, buffer, 0, BUF_SIZE);
    } catch (e) {
      if (e.code === 'EOF') { break; } // pipe

      if (useShell) {
        // Try reading via shell
        input = _readlineShell();
        if (typeof input === 'string') { break; }
      }

      // Give up...
      if (e.code === 'EAGAIN') { // EAGAIN, resource temporarily unavailable
        // util can't inherit Error.
        err = new Error('The platform doesn\'t support interactive reading from stdin');
        err.errno = e.errno;
        err.code = e.code;
      }
      if (display) { stdout.write('\n', encoding); } // Return from prompt line.
      throw err || e;
    }

    if (rsize === 0) { break; }
    input += buffer.toString(encoding, 0, rsize);
    if (/[\r\n]$/.test(input)) { break; }
  }
  stdin.pause();

  return input.trim();
}

function _readlineShell() {
  // piping via files instead of execSync (node v0.12+)
  var shellPath, args, shellStdout,
    pathStdout = getTempfile('readline-sync.stdout'),
    pathStatus = getTempfile('readline-sync.status'),
    pathDone   = getTempfile('readline-sync.done');

  if (process.platform === 'win32') {
    // The quote (") is escaped by node before parsed by shell. Then use ENV{Q}.
    shellPath = 'cmd.exe';
    args = ['/V:ON', '/S', '/C', '%Q%' + __dirname + '\\read.bat%Q% >%Q%' + pathStdout +
      '%Q% & (echo !ERRORLEVEL!)>%Q%' + pathStatus + '%Q% & (echo 1)>%Q%' + pathDone + '%Q%'];
  } else {
    shellPath = '/bin/sh';
    args = ['-c', '(' + shellPath + ' "' + __dirname + '/read.sh") >"' + pathStdout +
      '"; echo $? >"' + pathStatus + '"; echo 1 >"' + pathDone + '"'];
  }

  stdin.pause(); // re-start in child process
  require('child_process').execFile(shellPath, args, {env: {Q: '"'}});

  while (true) {
    if (fs.readFileSync(pathDone, {encoding: encoding}).trim() === '1') { break; }
  }
  if (fs.readFileSync(pathStatus, {encoding: encoding}).trim() === '0') {
    shellStdout = fs.readFileSync(pathStdout, {encoding: encoding});
  }

  fs.unlinkSync(pathStdout);
  fs.unlinkSync(pathStatus);
  fs.unlinkSync(pathDone);
  return shellStdout;
}

function getTempfile(name) {
  var path = require('path'), filepath, suffix = '', fd;
  tempdir = tempdir || require('os').tmpdir();

  while (true) {
    filepath = path.join(tempdir, name + suffix);
    try {
      fd = fs.openSync(filepath, 'wx');
    } catch (e) {
      if (e.code === 'EEXIST') {
        suffix++;
        continue;
      } else {
        throw e;
      }
    }
    fs.closeSync(fd);
    break;
  }
  return filepath;
}

// for dev
exports.useShellSet = function(use) { useShell = use; };

exports.setPrompt = function(newPrompt) {
  if (typeof newPrompt === 'string') {
    promptText = newPrompt;
  }
};

exports.setEncoding = function(newEncoding) {
  if (typeof newEncoding === 'string') {
    encoding = newEncoding;
  }
};

exports.prompt = function() {
  return _readlineSync(promptText);
};

exports.question = function(query) {
  return _readlineSync(typeof query === 'string' ? query : '');
};
